#ifdef VERTEX_SHADER

	layout (location = 0) in vec3 Position;

	out vec2 texCoord;

	void main()
	{
		gl_Position =  vec4(Position, 1);
		texCoord = vec2(Position.xy);
	} 
#endif

#ifdef FRAGMENT_SHADER

	#ifdef RenderPass_MotionBlur

		uniform int		u_numSamples;
		uniform vec2	u_viewPort;
		uniform mat4	u_invViewProjection;
		uniform mat4	u_prevViewProjection;		
		uniform sampler2DMS u_colorTex;
		uniform sampler2DMS u_depthTex;
		
		// Uniform for the tunnel effect
		uniform sampler2D u_tunnelTex;
		uniform int u_smWidth;
		uniform int u_smHeight;
		uniform float u_tunnelRot;
		uniform float u_tunnelTrans;
		uniform float u_alphaBlend;
		
		in vec2 texCoord;  


		out vec4 out_Color;

		vec4 singleSample(sampler2DMS texSampler, vec2 texCoords)
		{
			vec2 normC = (texCoords + 1) * 0.5;
			ivec2 screenCoords = ivec2(normC * u_viewPort);
			return texelFetch(texSampler, screenCoords, 0);
		}

		vec4 multisample4(sampler2DMS texSampler, vec2 texCoords)
		{
			vec2 normC = (texCoords + 1) * 0.5;
			ivec2 screenCoords = ivec2(normC * u_viewPort);

			vec4 value = vec4(0.0);
			for(int i = 0; i < 4; ++i)
				 value += texelFetch(texSampler, screenCoords, i) * 0.25;
			
			return value;
		}


		void main()
		{
			vec4 tunnel_Color;
			vec4 motion_Color;

			if (u_alphaBlend < 1.0f) {
				/*********************************************************************
				/* Algorithm from:
				/* http://http.developer.nvidia.com/GPUGems3/gpugems3_ch27.html
				/*********************************************************************/
				
				//non-linear depth
				float depthBufferZ = singleSample(u_depthTex, texCoord).x;

				vec4 ndcPos = vec4(texCoord.xy, depthBufferZ, 1); 
				vec4 worldPos = u_invViewProjection * ndcPos;  
				worldPos /= worldPos.w;  

				//get pixel velocity vector
				vec4 currentPos = ndcPos;
				vec4 previousPos = u_prevViewProjection * worldPos;  
				previousPos /= previousPos.w;    
				vec2 velocity = (currentPos - previousPos).xy / 10.0;  
 
				//multisample texture
				vec4 color = multisample4(u_colorTex, texCoord);
				vec2 actTexCoord = texCoord;
				actTexCoord += velocity;  
				for(int i = 1; i < u_numSamples; ++i, actTexCoord += velocity)  
				{   
					color += multisample4(u_colorTex, actTexCoord);  
				}
				
				motion_Color = color / u_numSamples;
			}
			
			if (u_alphaBlend > 0.0f) {
				// Idea from: http://benryves.com/tutorials/tunnel/4

				// Coordinate system is in the center of the screen
				float mX = gl_FragCoord.x - float(u_smWidth)/2;
				float mY = gl_FragCoord.y - float(u_smHeight)/2;

				// Calculate the angle
				float rot = atan(mY, mX);
			
				// Check if we ar in the 3rd and 4th quadrant
				if (mY < 0) rot = 2*M_PI + rot;

				// Normalize to 0-1 and add the animation uniform
				rot = 2*rot / (2*M_PI) + u_tunnelRot;

				// Calculate the depth value
				float depth = pow(mX, 2) + pow(mY, 2);
			
				// Normalize the depth to 0-1 and add the animation uniform
				float depthNorm = pow(float(u_smWidth)/2, 2) + pow(float(u_smHeight)/2, 2);
				float texDepth = mod(depthNorm/(70*depth) + u_tunnelTrans, 1.0f);

				// The last term is just to black out the end of the tunnel
				tunnel_Color = texture(u_tunnelTex, vec2(rot, texDepth)) * sqrt(depth/depthNorm);
			}

			out_Color = u_alphaBlend * tunnel_Color + (1 - u_alphaBlend) * motion_Color;
		}
	#endif
#endif