

vec3 sphereIntersect(vec4 position_eye, vec4 center_eye, float radius2)
{
	float dist2 = dot(position_eye - center_eye, position_eye - center_eye);
	if(dist2 > radius2)
		discard;

	//intersect line with sphere
	vec3 P2 = position_eye.xyz;
	vec3 P3 = center_eye.xyz;

	float a = dot(P2, P2);
	float b = 2.0 * dot(P2, -P3);
	float c = dot(-P3, -P3) - radius2;
	float radicand = b*b - 4.0*a*c; //should be >= 0 (intersection with eye-aligned plane..)
	float u = (-b - sqrt(radicand)) / (2.0*a);
	return u * P2;
}


#ifdef VERTEX_SHADER

	#ifdef RenderPass_Shaded


		layout(location = 0) in vec2 Position;

		uniform mat4 u_modelViewMat;
		uniform mat4 u_projectionMat;
		uniform float u_radius;
		uniform vec3 u_lightPosition;
		uniform mat4 u_viewMat;

		out vec4 position_eyeSpace;
		out vec4 center_eyeSpace;
		out vec4 lightPos_eyeSpace;

		void main()
		{           
			position_eyeSpace = u_modelViewMat * vec4(Position, 0.0, 1.0);
			center_eyeSpace = u_modelViewMat * vec4(0.0, 0.0, 0.0, 1.0);
			lightPos_eyeSpace = u_viewMat * vec4(u_lightPosition, 1.0);

			mat4 mvp = u_projectionMat * u_modelViewMat;
			gl_Position = mvp * vec4(Position, 0.0, 1.0);

		}
	#endif

	
	#ifdef RenderPass_Depth
	
		layout(location = 0) in vec3 Position;
		uniform mat4 u_modelMatrix;
		void main()
		{
			gl_Position = u_modelMatrix * vec4(Position, 1.0);
		}

	#endif

#endif
#ifdef GEOMETRY_SHADER
	
		layout(triangles, invocations = 6) in;
		layout(triangle_strip, max_vertices=18) out;
		
		uniform mat4 u_lightMatrices[6]; //world to light-clipspace

		out vec4 WS_pos_from_GS;

		void main() 
		{
			gl_Layer = gl_InvocationID;
			for(int tri_vert = 0; tri_vert < 3; ++tri_vert) 
			{
				WS_pos_from_GS = gl_in[tri_vert].gl_Position;
				gl_Position = u_lightMatrices[gl_Layer] * WS_pos_from_GS;
				EmitVertex();
			}
			EndPrimitive();

		}
#endif
#ifdef FRAGMENT_SHADER

	#ifdef RenderPass_Shaded
		
		uniform float u_radius;
		uniform mat4 u_viewMat;
		uniform mat4 u_invViewMat;
		uniform mat4 u_projectionMat;
		uniform sampler2D u_albedoTexture;
		uniform samplerCube u_cubeMatTexture;
		uniform samplerCube u_shadowCube;
		uniform vec3 u_lightPosition;
		uniform float u_distanceToFar;
		uniform vec4 u_ambientColor;
		uniform float u_specularTerm;
		uniform float u_lightIntensity;

		in vec4 position_eyeSpace;
		in vec4 center_eyeSpace;
		in vec4 lightPos_eyeSpace;

		out vec4 out_Color;

		void main()
		{

			vec3 x_eyeSpace = sphereIntersect(position_eyeSpace, center_eyeSpace, u_radius*u_radius);

			vec4 center_worldSpace = u_invViewMat * center_eyeSpace;
			vec4 x_worldSpace = u_invViewMat * vec4(x_eyeSpace, 1.0);
			vec3 lightToFragment = x_worldSpace.xyz - u_lightPosition;
			vec2 moments = texture(u_shadowCube, lightToFragment).rg;
			float distanceFromFragment = length(lightToFragment);

			float shadow = shadowContribution(moments, distanceFromFragment);

			out_Color = u_ambientColor * u_lightIntensity;

			//calc lighting:
			vec3 normal = normalize(x_eyeSpace - center_eyeSpace.xyz);
			vec3 L = normalize(lightPos_eyeSpace.xyz - x_eyeSpace);
			vec3 R = normalize(reflect(L, normal));
			float diffuse = max(dot(normal, L), 0.15);
			float specular = pow(max(dot(R, normalize(x_eyeSpace)), 0.0), u_specularTerm);
			vec4 normal_ws = u_invViewMat * vec4(normal, 0.0);
			vec4 viewVec_ws = u_invViewMat * vec4(x_eyeSpace, 0.0);
			vec3 lookup = reflect(viewVec_ws.xyz, normal_ws.xyz);
			vec4 albedo = texture(u_cubeMatTexture, lookup);
			
			out_Color += (diffuse*albedo + specular*albedo) * shadow * u_lightIntensity;

			//calc correct depth value
			vec4 x_clipSpace = u_projectionMat * vec4(x_eyeSpace, 1.0);
			float depthNDC = x_clipSpace.z / x_clipSpace.w;
			gl_FragDepth = ((gl_DepthRange.diff * depthNDC) + gl_DepthRange.near + gl_DepthRange.far) * 0.5f; 
		}


	#endif
	#ifdef RenderPass_Depth
			
			uniform vec3 u_lightPosition;
			uniform float u_distanceToFar;

			in vec4 WS_pos_from_GS;

			out vec2 out_moments;

			void main()
			{
				float WS_dist = distance(WS_pos_from_GS.xyz, u_lightPosition);
			
				float WS_dist_normalized = WS_dist / u_distanceToFar;
				out_moments = calcMoments(WS_dist);
				gl_FragDepth = WS_dist_normalized;
			}

	#endif

#endif
